/* This is a C++ file */
#include <stdio.h>
#include <string>
#include <limits.h>

using namespace std;

#define intsize (CHAR_BIT*sizeof(long)/2)

/****************************************/

int size = 128;

/* Klasa wielkich liczb */
class Vbig {
      public:
        // Deklaracja tablicy slow
        unsigned int v[128];
        // Konstruktory
        Vbig() {
                for (int i = 0; i <= (size - 1); i++)
                        v[i] = 0x0;
        } 
        Vbig(unsigned int x) {
                for (int i = 1; i <= (size - 1); i++)
                        v[i] = 0x0;
                v[0] = x;
        }
        // Operatory
        friend Vbig operator +(Vbig, unsigned int);
        friend Vbig operator +(Vbig, const Vbig &);
        friend Vbig operator -(Vbig, unsigned int);
        friend Vbig operator -(Vbig, const Vbig &);
        friend Vbig operator *(Vbig, unsigned int);
        friend Vbig operator *(const Vbig &, const Vbig &);
        friend Vbig operator /(Vbig, unsigned int);
        friend Vbig operator /(Vbig, Vbig);
        friend Vbig operator <<(Vbig, int);
        friend Vbig operator >>(Vbig, int);
        friend unsigned int operator %(Vbig, unsigned int);
        friend Vbig operator %(Vbig, Vbig);
        friend bool operator ==(const Vbig &, const Vbig &);
        friend bool operator !=(const Vbig &, const Vbig &);
        friend bool operator >(const Vbig &, const Vbig &);
        friend bool operator >=(const Vbig &, const Vbig &);
        friend bool operator <(const Vbig &, const Vbig &);
        friend bool operator <=(const Vbig &, const Vbig &);

        // Metody
        string str();
        void ch(char c[]);
        bool bit(int i);
        int blen();
        void set(int i);
        void fadd(Vbig & A);
        void fmul(unsigned int b);
        void fshl16();
        void fshr1();
};

/****************************************/

/* IMPLEMENTACJA */

/****************************************/

/* Szybkie porownanie (referencje) */
int CMP(const Vbig & A, const Vbig & B)
{
        int i = (size - 1);

        while ((A.v[i] == B.v[i]) && (i > 0))
                i--;
        if ((i == 0) && (A.v[0] == B.v[0]))
                return 0;
        else {
                if (A.v[i] > B.v[i])
                        return 1;
                if (A.v[i] < B.v[i])
                        return 2;
        };
        return 0;
}

/* Dodawanie cyfry do liczby */
Vbig operator +(Vbig A, unsigned int b)
{
        unsigned long C;
        int i = 0;

        C = b;
        do {
                C = C + A.v[i];
                A.v[i] = C & 0xFFFF;
                C >>= intsize;
                i++;
        }
        while ((C > 0) && (i <= (size - 1)));
        return A;
}

/* Dodawanie Vbig */
Vbig operator +(Vbig A, const Vbig & B)
{
        unsigned long C = 0;

        for (int i = 0; i <= (size - 1); i++) {
                C = C + A.v[i] + B.v[i];
                A.v[i] = C & 0xFFFF;
                C >>= intsize;
        }
        return A;
}

/* Odejmowanie cyfry od liczby */
Vbig operator -(Vbig A, unsigned int b)
{
        unsigned long C;
        unsigned long D = b;
        int i = 0;

        do {
                C = 1;
                C <<= intsize;
                C = C + A.v[i] - D;
                A.v[i] = C & 0xFFFF;
                C >>= intsize;
                if (C == 0) {
                        D = 1;
                } else {
                        D = 0;
                };
                i++;
        }
        while ((D != 0) && (i <= (size - 1)));
        return A;
}

/* Odejmowanie Vbig */
Vbig operator -(Vbig A, const Vbig & B)
{
        unsigned long c;
        int r = 0;
        Vbig z;

        if (A > B) {
                for (int i = 0; i <= (size - 1); i++) {
                        c = 65536;
                        c = c + A.v[i];
                        c = c - B.v[i];
                        c = c - r;
                        A.v[i] = c & 0xFFFF;
                        if (c < 65536)
                                r = 1;
                        else
                                r = 0;
                }
                return A;
        } else
                return z;
}

/* Mnozenie cyfra razy liczba */
Vbig operator *(Vbig A, unsigned int b)
{
        unsigned long C = 0;

        for (int i = 0; i <= (size - 1); i++) {
                C = C + A.v[i] * b;
                A.v[i] = C & 0xFFFF;
                C >>= intsize;
        }
        return A;
}

/* Mnozenie liczba razy liczba */
Vbig operator *(const Vbig & A, const Vbig & B)
{
        Vbig C;
        Vbig W;
        int i = (size - 1);

        while ((B.v[i] == 0) && (i > 0))
                i--;
        for (int j = i; j >= 0; j--) {
                W.fshl16();
                C = A;
                C.fmul(B.v[j]);
                W.fadd(C);
        }
        return W;
}

/* Dzielenie liczby przez cyfre */
Vbig operator /(Vbig A, unsigned int b)
{
        unsigned long C = 0;

        for (int i = size - 1; i >= 0; i--) {
                C = (C <<= intsize) + A.v[i];
                A.v[i] = C / b;
                C = C % b;
        }
        return A;
}

/* Dzielenie Big przez Big */
Vbig operator /(Vbig A, Vbig B)
{
        Vbig p;
        Vbig p1;
        Vbig W;
        int i = A.blen() - B.blen();

        if ((i > 0) && (B.blen() >= 0)) {
                B = B << i;
                for (int k = i; k >= 0; k--) {
                        p1.fadd(B);
                        if (CMP(A, p1) != 2) {
                                W.set(k);
                                p.fadd(B);
                        };
                        p1 = p;
                        B.fshr1();
                }
        }
        return W;
}

/* Modulo liczba przez cyfre */
unsigned int operator %(Vbig A, unsigned int b)
{
        unsigned long C = 0;

        for (int i = size - 1; i >= 0; i--) {
                C = (C <<= intsize) + A.v[i];
                A.v[i] = C / b;
                C = C % b;
        }
        return (C & 0xFFFF);
}

/* Modulo */
Vbig operator %(Vbig A, Vbig B)
{
        Vbig p = 0;
        Vbig p1 = 0;

        int i = A.blen();

        i = i - B.blen();

        if ((i > 0) && (B.blen() != -1)) {
                B = B << (i);
                for (int k = i; k >= 0; k--) {
                        p1.fadd(B);
                        if (!(p1 > A))
                                p.fadd(B);
                        p1 = p;
                        B.fshr1();
                }
                return A - p1;
        } else {
                if (A < B)
                        return A;
                else
                        return (A - B);
        }
}

/* Operator porownania */
bool operator ==(const Vbig & A, const Vbig & B)
{
        int i = 0;

        while ((A.v[i] == B.v[i]) && (i < size - 1)) {
                i++;
        };
        if ((i == size - 1) && (A.v[size - 1] == B.v[size - 1]))
                return true;
        else
                return false;
}

/* Operator porownania */
bool operator !=(const Vbig & A, const Vbig & B)
{
        return !(A == B);
}

/* Operator porownania */
bool operator >(const Vbig & A, const Vbig & B)
{
        int i = size - 1;

        while ((A.v[i] == B.v[i]) && (i > 0)) {
                i--;
        };
        if (A.v[i] > B.v[i])
                return true;
        else
                return false;
}

/* Operator porownania */
bool operator >=(const Vbig & A, const Vbig & B)
{
        return (A > B) || (A == B);
}

/* Operator porownania */
bool operator <(const Vbig & A, const Vbig & B)
{
        int i = size - 1;

        while ((A.v[i] == B.v[i]) && (i > 0))
                i--;
        if (A.v[i] < B.v[i])
                return true;
        else
                return false;
}

/* Operator porownania */
bool operator <=(const Vbig & A, const Vbig & B)
{
        return (A < B) || (A == B);
}

/* Szybkie dodawanie */
inline void Vbig::fadd(Vbig & A)
{
        unsigned long C = 0;

        for (int i = 0; i <= (size - 1); i++) {
                C += A.v[i] + v[i];
                v[i] = C & 0xFFFF;
                C >>= intsize;
        }
}

/* Szybkie mnozenie o slowo */
inline void Vbig::fmul(unsigned int b)
{
        unsigned long C = 0;

        for (int i = 0; i <= (size - 1); i++) {
                C += v[i] * b;
                v[i] = C & 0xFFFF;
                C >>= intsize;
        }
}

/* Szybkie SHL o 16 */
inline void Vbig::fshl16()
{
        for (int j = (size - 1); j >= 1; j--)
                v[j] = v[j - 1];
        v[0] = 0;
}

/* Szybkie SHR o 1 */
inline void Vbig::fshr1()
{
        unsigned long c = 0;

        for (int i = 0; i <= (size - 2); i++) {
                c = v[i + 1];
                c <<= intsize;
                c += v[i];
                c >>= 1;
                v[i] = c & 0xFFFF;
        }
        v[size - 1] >>= 1;
}

/* Operator SHL*/
Vbig operator <<(Vbig A, int b)
{
        int i, j;
        unsigned long c = 0;

        i = b >> 4;
        if (i > 0) {
                for (j = (size - 1); j >= i; j--)
                        A.v[j] = A.v[j - i];
                for (j = i - 1; j >= 0; j--)
                        A.v[j] = 0;
        };
        i = b & 0x000F;
        if (i > 0) {
                for (j = (size - 1); j >= 1; j--) {
                        c = A.v[j];
                        c <<= intsize;
                        c = c + A.v[j - 1];
                        c = c >> (intsize - i);
                        A.v[j] = c & 0xFFFF;
                };
                c = A.v[0];
                c = c << i;
                A.v[0] = c & 0xFFFF;
        }
        return A;
}

/* Operator SHR*/
Vbig operator >>(Vbig A, int b)
{
        int i, j;
        unsigned long c = 0;

        i = b >> 4;
        if (i > 0) {
                for (j = 0; j <= (size - 1) - i; j++)
                        A.v[j] = A.v[j + i];
                for (j = (size - 1) - i; j <= (size - 1); j++)
                        A.v[j] = 0;
        }
        i = b & 0x000F;
        if (i > 0) {
                for (j = 0; j <= (size - 2); j++) {
                        c = A.v[j + 1];
                        c <<= intsize;
                        c = c + A.v[j];
                        c >>= i;
                        A.v[j] = c & 0xFFFF;
                }
                c = A.v[size - 1];
                c >>= i;
                A.v[size - 1] = c & 0xFFFF;
        }
        return A;
}

/* Stan itego bitu */
bool Vbig::bit(int i)
{
        if ((v[i >> 4] & (1 << (i & 0x000F))) == 0)
                return false;
        else
                return true;
}

/* Ustawienie itego bitu */
void Vbig::set(int i)
{
        v[i >> 4] = (v[i >> 4] | (1 << (i & 0x000F)));
}

/* Dlugosc rozwiniecia binarnego */
int Vbig::blen()
{
        for (int i = size * 16 - 1; i >= 0; i--)
                if ((v[i >> 4] & (1 << (i & 0x000F))) != 0)
                        return i;
        return -1;
}

/* Konwersja lancucha na Big */
void Vbig::ch(char c[])
{
        int i = 0;
        Vbig b;

        do {
                b = b * 10;
                switch (c[i]) {
                case '1':
                        {
                                b = b + 1;
                                break;
                        };
                case '2':
                        {
                                b = b + 2;
                                break;
                        };
                case '3':
                        {
                                b = b + 3;
                                break;
                        };
                case '4':
                        {
                                b = b + 4;
                                break;
                        };
                case '5':
                        {
                                b = b + 5;
                                break;
                        };
                case '6':
                        {
                                b = b + 6;
                                break;
                        };
                case '7':
                        {
                                b = b + 7;
                                break;
                        };
                case '8':
                        {
                                b = b + 8;
                                break;
                        };
                case '9':
                        {
                                b = b + 9;
                                break;
                        };
                }
                i++;
        }
        while (c[i] != '\0');
        for (int j = 0; j < size; j++)
                v[j] = b.v[j];
}

/* Konwersja Big na string */
string Vbig::str()
{
        string s = "";
        unsigned int z;
        Vbig B;
        Vbig A;

        for (int j = 0; j < size; j++)
                A.v[j] = v[j];
        if (A == B)
                s = "0";
        while (A != B) {
                z = A % 10;
                A = A / 10;
                switch (z) {
                case 0:
                        s = "0" + s;
                        break;
                case 1:
                        s = "1" + s;
                        break;
                case 2:
                        s = "2" + s;
                        break;
                case 3:
                        s = "3" + s;
                        break;
                case 4:
                        s = "4" + s;
                        break;
                case 5:
                        s = "5" + s;
                        break;
                case 6:
                        s = "6" + s;
                        break;
                case 7:
                        s = "7" + s;
                        break;
                case 8:
                        s = "8" + s;
                        break;
                case 9:
                        s = "9" + s;
                        break;
                }
        }
        return s;
}

void contr(Vbig A)
{
        unsigned int c;

        for (int i = 0; i <= size - 1; i++) {
                c = A.v[i];
                printf(" %d", c);
        }
        printf("\n");
}

void fmulmod(Vbig & A, const Vbig & B, const Vbig & N)
{
        Vbig C;
        Vbig W;
        Vbig p1, p;
        int i = (size - 1);

        while ((B.v[i] == 0) && (i > 0))
                i--;
        for (int j = i; j >= 0; j--) {
                W.fshl16();
                C = A;
                C.fmul(B.v[j]);
                W.fadd(C);
        }
        int b_w;

        for (i = size * 16 - 1; i >= 0; i--)
                if ((W.v[i >> 4] & (1 << (i & 0x000F))) != 0) {
                        b_w = i;
                        break;
                }

        if ((b_w == 0) && (W.v[0] & 0x1 == 0))
                b_w = -1;
        int b_n;

        for (i = size * 16 - 1; i >= 0; i--)
                if ((N.v[i >> 4] & (1 << (i & 0x000F))) != 0) {
                        b_n = i;
                        break;
                }

        if ((b_n == 0) && (N.v[0] & 0x1 == 0))
                b_n = -1;
        C = N;
        i = b_w - b_n;
        if ((i > 0) && (b_n > -1)) {
                C = C << (i);
                for (int k = i; k >= 0; k--) {
                        p1.fadd(C);
                        if (!(p1 > W))
                                p.fadd(C);
                        p1 = p;
                        C.fshr1();
                }
                A = W - p1;
        } else {
                if (W < N)
                        A = W;
                else
                        A = (W - N);
        }
}
/****************************************/

